/*
 * Copyright  Micromega Corporation 2002-2007. All rights reserved.
 */
import com.micromegacorp.math.v3_spi.*;   // (use one of the uM-FPU packages)
//import com.micromegacorp.math.v3_i2c.*;
import stamp.core.*;

/**
 * @file    Graph.java
 * @target  Javelin Stamp
 *
 * This program demonstrates how to use the uM-FPU V3 floating point coprocessor
 * connected to the Javelin Stamp using either a 2-wire SPI or I2C interface.
 * It draws a simple graph of the sine and cosine functions in the terminal
 * window.
 *
 * @author Cam Thompson, Micromega Corporation
 * @version V1.0
 *    March 20, 2006
 *    - original version
 */

public class Graph {

  //-------------------- uM-FPU register definitions --------------------------

  final static int Angle = 1;           // angle
  final static int Point = 2;           // point value

  // string buffer
  static StringBuffer sbuf = new StringBuffer(30);
  static String underline = "---------------------";

  //-------------------- main routine -----------------------------------------

  public static void main() {

    // display program name
    System.out.println("\u0010Graph");

    // reset the uM-FPU and print version string
    if (!Fpu.reset()) {
      System.out.println("uM-FPU not responding.");
      return;
    }
    else
      System.out.println(Fpu.version());

    // graph sine from 0 to 360 degrees (2*PI radians)
    System.out.println("\r\nGraph of sine");
    System.out.println("-1        0        1");
    System.out.println(underline);
    for (int i = 0; i <= 20; i++) {
      Fpu.startWrite();
      Fpu.write(Fpu.SELECTA, Angle, Fpu.LOADPI, Fpu.FSET0);
      Fpu.write(Fpu.FMULI, 2*i, Fpu.FDIVI, 20);
      Fpu.write(Fpu.SIN);
      Fpu.stop();
      graph();
    }

    // graph cosine from 0 to 360 degrees (2*PI radians)
    System.out.println("\r\nGraph of sine");
    System.out.println("-1        0        1");
    System.out.println(underline);
    for (int i = 0; i <= 20; i++) {
      Fpu.startWrite();
      Fpu.write(Fpu.SELECTA, Angle, Fpu.LOADPI, Fpu.FSET0);
      Fpu.write(Fpu.LOADBYTE, 2*i, Fpu.FMUL0);
      Fpu.write(Fpu.LOADBYTE, 20, Fpu.FDIV0, Fpu.COS);
      Fpu.stop();
      graph();
    }
  }

  //-------------------- graph ------------------------------------------------

  // plot the current Angle value on a graph
  static void graph() {
    // calculate point on graph for value

    Fpu.startWrite();
    Fpu.write(Fpu.SELECTA, Point, Fpu.FSET, Angle);
    Fpu.write(Fpu.FMULI, 10, Fpu.ROUND, Fpu.FIX);
    Fpu.wait();
    Fpu.startWrite();
    Fpu.write(Fpu.LREADWORD);
    int point = Fpu.readWord();

    // plot the point on the graph
    char ch;
    for (int i = -10; i <= 10; i++) {
      if (i == point) ch = '*';
      else if (i == 0) ch = '|';
      else ch = ' ';
      System.out.print(ch);
    }
    // display the value
    System.out.print(' ');
    System.out.println(Fpu.floatFormat(Angle, 85));
  } // end graph

} // end class